parserwrapper

package
v1.0.1 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Mar 1, 2026 License: MIT Imports: 6 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

This section is empty.

Types

type ParserWrapepr

type ParserWrapepr struct {
	// contains filtered or unexported fields
}

ParserWrapepr is a wrapper for the swag annotations and source code parsers.

It uses the both parsers to read the complete information about the annotated functions and the future API.

func NewParserWrapper

func NewParserWrapper() *ParserWrapepr

NewParserWrapper returns a new Parser.

func (*ParserWrapepr) GetInfo

func (p *ParserWrapepr) GetInfo() models.ParsingInfo

GetInfo returns the complete parsing information.

func (*ParserWrapepr) Parse

func (p *ParserWrapepr) Parse(pkgPath, mainFname, handlerName string) error

Parse runs parsing.

Runs the swag annotations and source code parsers and joins their results into the complete parsing information.

Jump to